

#define MIN_ITERATION_DEFAULT 0
#define MAX_ITERATION_DEFAULT 200000
#define CHECKPOINT_FREQUENCY_DEFAULT 200

/* Problem parameters */

// Scale coefficient selects problem size
#ifndef SCALE
    #define SCALE (1.0f)
#endif //SCALE

// Height of dam
#define T (0.6f * SCALE)

// Width of dam
#define L (1.2f * SCALE)

// Size of tank
#define B (3.22f * SCALE)

// Resolution
#define DELTA (0.01f)
#define H (0.94f*DELTA*1.4142135623f)

// Pair estimate
#define PAIR_ESTIMATE 300

typedef int64_t int_t;
typedef uint64_t uint_t;
#define INT_MACRO_MPI MPI_LONG
#define ISTR PRId64
typedef float real_t;
#define REAL_MACRO_MPI MPI_DOUBLE

#define RADIUS (3.0f * H) // scale_k * H
#define BUCKET_RADIUS (1*RADIUS)

static const real_t
    D_tank = 1.0f * SCALE,
    scale_k = 3.0f,
    density = 1e3f,
    dt = 1e-4f,
    sos = 50.0f;

static const int_t
    free_surface = 30;

typedef struct {
    int_t
        idx,
        interactions;
    real_t
        x[2],   // 2D position
        v[2],   // 2D velocity
        mass,   // Mass
        rho,    // Density
        p,      // Pressure
        type,   // Type of particle
        hsml;   // Related to resolution (not fully understood yet)

    // Differentials
    real_t
        indvxdt[2],
        exdvxdt[2],
        dvx[2],
        drhodt;
    // Density correction
    real_t
        avrho,
        w_sum;
//#if BUCKET
    //int_t bucket_x;
    //int_t bucket_y;
    int_t bucket_id;
    int_t local_idx;
//#endif //BUCKET
} particle_t;

// Pairwise interaction
typedef struct {
    int_t i, j;     // Which particles interact?
//#ifdef BUCKET // cannot be used with GPU
//    particle_t* ip;
//    particle_t* jp;
//#endif //BUCKET
    real_t
        r,          // Distance between particles (Euclid)
        q,          // Distance normalized to H (resolution)
        w,          // TODO: consult paper for meaning of this
        dwdx[2];    // Influence on velocity
} pair_t;

typedef struct bucket_t bucket_t;
/* A bucket is cell list of particles */
struct bucket_t {
    uint_t* idxs;
    int_t total;
};
#define INITIAL_MAX_BUCKET 96

#define BID(X,Y) (Y + N_BUCKETS_Y * X)

//#define N_BUCKETS_X ((int_t)(ceil(((subdomain[1]-subdomain[0])+2*RADIUS) / BUCKET_RADIUS))) //RADIUS is the maximum distance (from each sides of the subdomain boundaries) where the mirror particles can be located. RADIUS>1.55*H (otherwise replace RADIUS by 1.55*H when computing N_BUCKETS_X)
//#define N_BUCKETS_Y ((int_t)(ceil(((1.5f*T)+1.55f*H) / BUCKET_RADIUS))) //1.55*H is the boundary used when generating virtual particles

int_t n_buckets_x;
int_t n_buckets_y;

#define CAP_INCREMENT 4096

// Newly added convinience & speedup macros
#define SQUARE(x) (((x)*(x)))
#define POW2(x) (((x)*(x)))
#define POW3(x) (((x)*(x)*(x)))
#define POW4(x) (((x)*(x)*(x)*(x)))
#define POW5(x) (((x)*(x)*(x)*(x)*(x)))
#define POW6(x) (((x)*(x)*(x)*(x)*(x)*(x)))
#define POW7(x) (((x)*(x)*(x)*(x)*(x)*(x)*(x)))
#define ceilDiv(n, div) ((((n) + div - 1)/div))

#define MIN(X, Y) ((X) < (Y) ? (X) : (Y))
#define MAX(X, Y) ((X) > (Y) ? (X) : (Y))

// Physics Macros - particle list must be called list
#define X(k)        ((list[k]).x[0])
#define Y(k)        ((list[k]).x[1])
#define VX(k)       ((list[(k)]).v[0])
#define VY(k)       ((list[(k)]).v[1])
#define M(k)        ((list[(k)]).mass)
#define RHO(k)      ((list[(k)]).rho)
#define P(k)        ((list[(k)]).p)
#define TYPE(k)     ((list[(k)]).type)
#define HSML(k)     ((list[(k)]).hsml)
#define INTER(k)    ((list[(k)]).interactions)

#define INDVXDT(k,i)    ((list[(k)]).indvxdt[(i)])
#define EXDVXDT(k,i)    ((list[(k)]).exdvxdt[(i)])
#define DVX(k,i)        ((list[(k)]).dvx[(i)])
#define DRHODT(k)       ((list[(k)]).drhodt)

#define AVRHO(k)    ((list[(k)]).avrho)
#define WSUM(k)     ((list[(k)]).w_sum)