#include <thrust/device_ptr.h>
#include <thrust/sort.h>
#include <thrust/execution_policy.h>
#include <thrust/system_error.h>

typedef struct {
    int64_t
        idx,
        interactions;
    double
        x[2],   // 2D position
        v[2],   // 2D velocity
        mass,   // Mass
        rho,    // Density
        p,      // Pressure
        type,   // Type of particle
        hsml;   // Related to resolution (not fully understood yet)

    // Differentials
    double
        indvxdt[2],
        exdvxdt[2],
        dvx[2],
        drhodt;
    // Density correction
    double
        avrho,
        w_sum;
//#if BUCKET
    //int_t bucket_x;
    //int_t bucket_y;
    int64_t bucket_id;
    int64_t local_idx;
//#endif //BUCKET
} particle_t;

void sort_cells(uint* particle_hash, particle_t* d_values, int n_total) {
	//thrust::device_ptr<uint> p_h (particle_hash);
    //thrust::device_ptr<uint> d_v (d_values);
    start_of_try:
    try {
	    thrust::stable_sort_by_key(thrust::device,
	    	particle_hash,
	    	particle_hash + n_total,
	    	d_values);
    } catch(thrust::system_error &e) {
    	//printf("Error: %s \n",e.what());
    	// Ugly fix for if running on one gpu, because thrust doesn't like to share
    	goto start_of_try;
    }
}