#ifdef __APPLE__
#define _GNU_C_SOURCE
#elif __linux__
#define _XOPEN_SOURCE 500
#endif

#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <inttypes.h>
#include <stdbool.h>
#include <string.h>
#include <math.h>
#include <mpi.h>
#include <omp.h>
#include <getopt.h>
#include <unistd.h>
#include "defs.h"

/* Global state variables, definitions are in sph.c */
extern int size, rank, east, west;
extern int_t n_global_field, n_field;

/* Setup and takedown */
void initialize ( void );
void finalize ( void );

// Hash table interface (in particle_hashtab.c)
void particles_init ( void );
void particles_finalize ( void );

// Insert a pointer to a particle
void insert_particle ( particle_t *p );
// Remove a pointer to a particle
void remove_particle ( particle_t *p );
// Lookup particle pointer by index
void lookup_particle ( int_t index, particle_t **p );
// Start from particle pointer, serialize contents of table (pointers)
void list_particles ( particle_t **list_point );
// Start from particle pointer, serialize particle data
void marshal_particles ( particle_t *list_point );
// Extract actual particles from list to hash tab
void unmarshal_particles ( particle_t *list, int_t count );
// Count the number of local actuals
int_t n_particles ( void );

// I/O and auxiliary stuff
void dump_state ( char *filename );
void resize_list ( int_t required );
void resize_pair_list ( int_t new_cap );
void collect_checkpoint ( void );
void write_checkpoint ( char *filename );
void restart_checkpoint ( int_t iteration );
void options ( int argc, char **argv );
void free_buckets(bucket_t* bucket);
void print_timing(const char* full_string, const char* short_string, const double value);

// Parts of the solver
void generate_virtual_particles ( void );
void create_pairs(int bx, int by, bucket_t** buckets,
                  particle_t* particle, int_t* n_pairs,
                  int_t* interactions);
void create_pairs_old(int bx, int by, bucket_t* buckets,
                      particle_t* particle, int_t* n_pairs);
#ifdef FILL_BUCKETS_LOCK
static void fill_buckets2(bucket_t**, int, omp_lock_t* lock);
#else
static void fill_buckets1(bucket_t**, int);
#endif //FILL_BUCKETS_LOCK


// MPI communication
void border_exchange( void );
void migrate_particles ( void );

// Extern cuda functions from kernel.cu
extern void print_cuda_info();
extern void initialize_cuda();
extern void finalize_cuda();
